library(readr)
library("rjson")
library(stringr)
library(ggplot2)
library(dplyr)
library(tidyr)
library(tidyverse)
library(gridExtra)

## read data
df_tsv<-readr::read_tsv("D:\\Dropbox\\tudelft\\Course\\R\\data\\views_downloads.tsv")
data_json <- rjson::fromJSON(file="D:\\Dropbox\\tudelft\\Course\\R\\data\\datasets.json")

## save data in the dataframe
dfbig <- data.frame()
embargo <- data.frame()
for (i in 1:length(data_json)){
  if (length(data_json[[i]])==18){
    x <- as.data.frame(do.call(rbind, data_json[i]))
    x1 <- data.frame(do.call(rbind,x$timeline))
    row.names(x1) <- NULL
    matching_rows <- which(df_tsv$uuid == x$uuid)
    if (length(matching_rows) == 0){
      x2 <- data.frame(views = c(0), downloads = c(0))
      print("no views record")
      print(i)
    }
    else{
      x2 <- df_tsv[matching_rows,c("views", "downloads")]
    }
    x <- cbind(x,x1,x2)
    dfbig <- rbind(dfbig, x)
    #print(i)
  }
  else{
    x <- as.data.frame(do.call(rbind, data_json[i]))
    embargo <- rbind(embargo, x)
    print(i)
  }
}

# adjust date format
date_pattern <- "\\d{4}-\\d{2}-\\d{2}" 
year_pattern <- "\\d{4}"
month_pattern <- "\\d{4}-\\d{2}"
day <- data.frame(dfbig[c("posted")])
dates <- as.Date(str_extract(day[[1]], date_pattern))
embargo_dates <- as.Date(str_extract(embargo[[1]], date_pattern))
years <- str_extract(day[[1]], year_pattern)
months <- str_extract(day[[1]], month_pattern)
months_all <- as.integer(format(dates, "%m"))
row.names(dates) <- NULL
row.names(months) <- NULL
row.names(years) <- NULL

# creat data frame with useful message
dfuse <- cbind(dfbig[c('uuid','url','defined_type','defined_type_name','group_id','views','downloads')],months,years,months_all,dates)
dfuse$group_id <- lapply(dfuse$group_id, function(x) ifelse(is.null(x), "Unknown", x))
dfuse <- dfuse %>%
  mutate(group_id = factor(unlist(dfuse$group_id)))

### use the posted date information and the counts
## format adjustment for Fig.1, Fig.2
b<- dfuse %>%
  group_by(months,defined_type,defined_type_name) %>%
  count() 

d<- embargo %>%
  group_by(embargo_date) %>%
  count() 
df_no_na_months <- na.omit(b)

chapters = c(0,0,0,0,0,0,0)
df2 = frame_data()
df2 <- cbind(d, chapters)
colnames(df2) <- c("months", "n", "defined_type_name")
df2$months <- sapply(df2$months, as.character)
df2$defined_type_name <- lapply(df2$defined_type_name, list)

df3 = frame_data()
df3 = rbind(df_no_na_months,df2)

# plot Figure 1
dd = data.frame(df3[1:194, ])
dd$defined_type_name <- sapply(dd$defined_type_name, as.character)
ee = data.frame(df3[195:201, ])
ee$defined_type_name <- sapply(ee$defined_type_name, as.character)
ggplot() + 
  geom_bar(data = dd,aes(x = factor(months), y = n, fill = defined_type_name),  stat = "identity") +
  labs(title = "Bar Chart of all Month by Defined Type", x = "all Months", y = "Count") +
  theme(axis.text.x = element_text(angle = 90, hjust = 1)) +
  geom_point(data = ee,aes(x = factor(months), y = n))
  labs(title = "Bar Chart of all Month by Defined Type", x = "all Months", y = "Count") +
  theme(axis.text.x = element_text(angle = 90, hjust = 1))

# plot Figure 2
selected <- dfuse
selected$dates <- as.Date(selected$dates)
aa<- selected %>%
  filter(dates>"2020-09-30")%>%
  group_by(months_all,defined_type_name) %>%
  count()

aa$defined_type_name <- sapply(aa$defined_type_name, as.character)
ggplot(aa, aes(x = factor(months_all), y = n, fill = defined_type_name)) + 
  geom_bar(stat = "identity") +
  labs(title = "Bar Chart of Month by Defined Type (Date > 2020-09-30)", x = "Months", y = "Count") +
  theme(axis.text.x = element_text(angle = 0, hjust = 1), legend.title = element_text(size=10, face="bold"))


### use group_id
library(wordcloud)
library(RColorBrewer)
#library(tm) # for text mining
#library(SnowballC) # for text stemming

# plot Figure 3 
df_groupid <- dfuse %>%
  group_by(group_id) %>%
  count()
View(df_groupid)
wordcloud(df_groupid$group_id, df_groupid$n, 
          colors = brewer.pal(8, "Accent"), scale = c(5, 0.5))

# adjust data for figure 4
c2<- dfuse %>%
  group_by(years,group_id) %>%
  count()
c2$group_id <- as.character(c2$group_id)
unique_groups <- unique(c2$group_id)
mapping <- setNames(seq_along(unique_groups), unique_groups)
c2$group_id <- lapply(c2$group_id, function(x) mapping[x])
c2$group_id <- factor(unlist(c2$group_id))

# plot Figure 4 
ggplot(c2, aes(x = years, y = n, fill = group_id)) +
  geom_bar(stat = "identity", position = "dodge") +
  labs(x = "Year", y = "Count", fill = "Group ID") +
  theme_minimal() +
  scale_fill_discrete(name = "Group ID")

## work for Figure 5
# Gather the data for plotting
summary_stats <- dfuse %>%
  group_by(group_id) %>%
  summarise(
    mean_views = mean(views, na.rm = TRUE),
    sd_views = sd(views, na.rm = TRUE),
    mean_downloads = mean(downloads, na.rm = TRUE),
    sd_downloads = sd(downloads, na.rm = TRUE)
  )
View(summary_stats)

# Replace missing values with 0 during data gathering
gathered_data <- summary_stats %>%
  gather(key = "variable", value = "value", -group_id) %>%
  mutate(
    category = ifelse(grepl("views", variable), "views", "downloads"),
    variable = ifelse(grepl("mean", variable), "mean", "std")
  )
gathered_data <- gathered_data %>%
  spread(key = variable, value = value) %>%
  rename(mean = mean, std = std)
gathered_data$mean[is.na(gathered_data$mean)] <- 0
gathered_data$std[is.na(gathered_data$std)] <- 0

# Create a bar plot with error bars
ggplot(gathered_data, aes(x = group_id, y = mean, fill = category)) +
  geom_bar(stat = "identity", position = position_dodge(width = 0.8), width = 0.6) +
  geom_errorbar(aes(ymin = mean - std, ymax = mean + std), position = position_dodge(width = 0.8), width = 0.25, alpha = 0.3) +
  labs(x = "Group ID", y = "Counts", fill = " ") +
  scale_fill_manual(values = c("views" = "blue", "downloads" = "green")) +
  theme_minimal() + 
  theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust = 1))


### use url information to extract data categories 
### try out to collect information from url for Figure 6 and 7
library(httr)
library(RCurl)
library(rvest)
library(xml2)

df_cat <- data.frame(matrix(ncol=5,nrow=1))
df_cat <- df_cat[-1,]
for (i in 1:nrow(dfuse)){
  ## extract the categories in the url
  print(i)
  url <- as.character(dfuse[i,"url"])
  response <- GET(url)
  text <- content(response, "text")
  categories_info <- str_extract(text, '"categories":\\s*\\[([^]]+)\\]')
  titles <- str_extract_all(categories_info, '"title":\\s*"([^"]+)"')[[1]]
  for (ii in 1:length(titles)){
    matched_title <- str_match(titles[ii], '"title": "([^"]+)"')
    if (!is.na(matched_title[2])) {
      extracted_title <- matched_title[2]
      # Remove escape characters
      extracted_title <- gsub('\\\\', '', extracted_title)
    } else {
      print("Title not found.")
    }
    
    ## add to df
    xx <- c("uuid"=as.character(dfuse[i,"uuid"]), 
            "url"=as.character(dfuse[i,"url"]), 
            "category"=extracted_title,
            "views"=dfuse[i,"views"],
            "downloads"=dfuse[i,"downloads"])
    df_cat <- rbind(df_cat,xx)
  }
}
colnames(df_cat) <- c("uuid", "url", "category","views","downloads")


# plot Figure 6 
library(wordcloud2) 
df_groupid <- df_cat %>%
  group_by(category) %>%
  count()
View(df_groupid)

wordcloud2(data=df_groupid, size=0.4, color='random-dark')# for Figure 6.1
wordcloud2(data=df_groupid, size=0.6, color='random-dark')# for Figure 6.2

## for Figure 7
# adjust data for Figure 7
df_cat$views<- as.numeric(df_cat$views)
df_cat$downloads<- as.numeric(df_cat$downloads)

summary_stats <- df_cat %>%
  group_by(category) %>%
  summarise(
    mean_views = mean(views, na.rm = TRUE),
    sd_views = sd(views, na.rm = TRUE),
    mean_downloads = mean(downloads, na.rm = TRUE),
    sd_downloads = sd(downloads, na.rm = TRUE)
  ) 
View(summary_stats)

summary_stats <- summary_stats %>%
  filter(mean_views>1000 & mean_downloads>1000)

# Replace missing values with 0 during data gathering
gathered_data <- summary_stats %>%
  gather(key = "variable", value = "value", -category) %>%
  mutate(
    category_1 = ifelse(grepl("views", variable), "views", "downloads"),
    variable = ifelse(grepl("mean", variable), "mean", "std")
  )

gathered_data <- gathered_data %>%
  spread(key = variable, value = value) %>%
  rename(mean = mean, std = std)

# plot Figure 7
ggplot(gathered_data, aes(x = category, y = mean, fill = category_1)) +
  geom_bar(stat = "identity", position = position_dodge(width = 0.8), width = 0.6) +
  geom_errorbar(aes(ymin = mean - std, ymax = mean + std), position = position_dodge(width = 0.8), width = 0.25, alpha = 0.3) +
  labs(x = "Category", y = "Counts", fill = " ") +
  scale_fill_manual(values = c("views" = "blue", "downloads" = "green")) +
  theme_minimal() + 
  theme(axis.text.x = element_text(angle = 45, hjust = 1))
